/* ----------------------------------------------------------------------------
 * pbb_lib.c
 * main funtions of pbb library
 *
 * Copyright 2002 Matthias Grimm (joker@cymes.de).
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version
 * 2 of the License, or (at your option) any later version.
 * ----------------------------------------------------------------------------*/

#include <stdio.h>
#include <unistd.h>
#include <string.h>
#include <syslog.h>

#include "pbb.h"

struct libbase libdata;     /* library basic data */

/*---------------------------------------------------------------------
 * void init_libpbbuttons ()
 *
 * This function initialises the basic library structure and
 * have to be called before any library funcion is used.
 *---------------------------------------------------------------------*/

void
init_libpbb ()
{
	struct libbase *base = &libdata;
	int n;

	base->serverport = 0;
	base->msgport = 0;
	base->ptagcount = 0;
	base->count = 0;
	base->daemon = 0;
	base->child = 0;
	base->filtermode = 0;  /* no filtering by default */
	base->uid = geteuid();   /* uid from cruuent process */
	base->pidfile[0] = '\0';
	taglist_init (base->ptags);

	for (n=0; n < MAXCLIENTS; n++)
		base->client[n] = 0;
}

void
exit_libpbb ()
{
	struct libbase *base = &libdata;

	if (base->child)
		unlink (base->pidfile);
}

/* This is the old method to detach a program from terminal
 * It is replaced by the new method with prepare_daemon()
 * and create_daemon() and should not be used in new programs.
 */
void
detach_process (char *name)
{
	struct libbase *base = &libdata;

	if (fork() != 0)
		exit (0);      /* exit parent process */
	base->daemon = 1;
	base->child = 1;
	(void) setsid();
	openlog (name, 0, LOG_DAEMON);
}

/* This function is for compatibility reasons. It should
 * be called as early as possible to show the library that
 * this process becomes a daemon so that error messages
 * could be redirected into a logfile instead of printing
 * them to console. It also saves the pidfile name for
 * later use.
 */
int
prepare_daemon (char *name, char *pidfile, int flags)
{
	struct libbase *base = &libdata;
	int err;

	err = check_devorfile(pidfile, TYPE_FILE);
	/* If the caller set PBBDF_FORCE, if wouldn't matter
	   if the pidfile already exists. The caller takes
	   care about it.
	 */
	if (err == 0 && flags == PBBDF_FORCE)
		err = E_NOEXIST;

	if (err == E_NOEXIST) {
		strncpy (base->pidfile, pidfile, sizeof(base->pidfile));
		base->daemon = 1;
		openlog (name, 0, LOG_DAEMON);
	}
	return err;
}

int
create_daemon ()
{
	struct libbase *base = &libdata;
	int pid = 0;
	FILE *fh;

	if (base->daemon == 1) {
		if ((pid = fork()) == 0) {
			base->child = 1;        /* this is the child now */
			fclose(stdin);
			fclose(stdout);
			fclose(stderr);
			chdir("/");
			setsid();
		} else {           /* the parent process have to save the pid */
			if ((fh = fopen (base->pidfile, "w")) != 0) {
				fprintf (fh, "%i\n", pid);
				fclose (fh);
			}
		}
	}
	return pid;
}
