/* --------------------------------------------------------------------------
 * module_test.c
 * code for test module
 *
 * Copyright 2002 Matthias Grimm (joker@cymes.de).
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version
 * 2 of the License, or (at your option) any later version.
 * -------------------------------------------------------------------------*/
#ifdef DEBUG

#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#include <stdio.h>
#include <stdlib.h>
#include <errno.h>
#include <string.h>
#include <sys/time.h>

#include <pbb.h>

#include "gettext_macros.h"
#include "input_manager.h"
#include "module_peep.h"
#include "module_pmac.h"
#include "support.h"

static struct tagtable tagtab[] = {
  { "TAG_SKIP",              TAG_SKIP},
  { "TAG_SAVECONFIG",        TAG_SAVECONFIG},
  { "TAG_USERALLOWED",       TAG_USERALLOWED},
  { "TAG_AUTORESCAN",        TAG_AUTORESCAN},
  { "TAG_VERSION",           TAG_VERSION},
  { "TAG_KEYCODE",           TAG_KEYCODE},
  { "TAG_KEYREPEAT",         TAG_KEYREPEAT},
  { "TAG_MODIFIER",          TAG_MODIFIER},
  { "TAG_MOUSEBUTTONS",      TAG_MOUSEBUTTONS},
  { "TAG_MOUSERELX",         TAG_MOUSERELX},
  { "TAG_MOUSERELY",         TAG_MOUSERELY},
  { "TAG_MOUSEWHEEL",        TAG_MOUSEWHEEL},
  { "TAG_BACKLIGHTMAX",      TAG_BACKLIGHTMAX},
  { "TAG_BACKLIGHTLEVEL",    TAG_BACKLIGHTLEVEL},
  { "TAG_COVERSTATUS",       TAG_COVERSTATUS},
  { "TAG_TIMEREMAINING",     TAG_TIMEREMAINING},
  { "TAG_TIMECHANGED",       TAG_TIMECHANGED},
  { "TAG_POWERSOURCE",       TAG_POWERSOURCE},
  { "TAG_POWERCHANGED",      TAG_POWERCHANGED},
  { "TAG_REQUESTSLEEP",      TAG_REQUESTSLEEP},
  { "TAG_SLEEPSUPPORTED",    TAG_SLEEPSUPPORTED},
  { "TAG_PREPAREFORSLEEP",   TAG_PREPAREFORSLEEP},
  { "TAG_WAKEUPFROMSLEEP",   TAG_WAKEUPFROMSLEEP},
  { "TAG_PMUDEVICE",         TAG_PMUDEVICE},
  { "TAG_ADBDEVICE",         TAG_ADBDEVICE},
  { "TAG_TPMODEUPKEY",       TAG_TPMODEUPKEY},
  { "TAG_TPMODEUPMOD",       TAG_TPMODEUPMOD},
  { "TAG_TPMODEDOWNKEY",     TAG_TPMODEDOWNKEY},
  { "TAG_TPMODEDOWNMOD",     TAG_TPMODEDOWNMOD},
  { "TAG_TPMODE",            TAG_TPMODE},
  { "TAG_KBDMODE",           TAG_KBDMODE},
  { "TAG_REPLACEPMUD",       TAG_REPLACEPMUD},
  { "TAG_BRIGHTNESSUPKEY",   TAG_BRIGHTNESSUPKEY},
  { "TAG_BRIGHTNESSUPMOD",   TAG_BRIGHTNESSUPMOD},
  { "TAG_BRIGHTNESSDOWNKEY", TAG_BRIGHTNESSDOWNKEY},
  { "TAG_BRIGHTNESSDOWNMOD", TAG_BRIGHTNESSDOWNMOD},
  { "TAG_BRIGHTNESS",        TAG_BRIGHTNESS},
  { "TAG_BRIGHTNESSOP",      TAG_BRIGHTNESSOP},
  { "TAG_BRIGHTNESSMAX",     TAG_BRIGHTNESSMAX},
  { "TAG_FRAMEBUFFERDEVICE", TAG_FRAMEBUFFERDEVICE},
  { "TAG_BLANKFRAMEBUFFER",  TAG_BLANKFRAMEBUFFER},
  { "TAG_FADINGSPEED",       TAG_FADINGSPEED},
  { "TAG_DIMFULLYDARK",      TAG_DIMFULLYDARK},
  { "TAG_MIXERDEVICE",       TAG_MIXERDEVICE},
  { "TAG_VOLUMEUPKEY",       TAG_VOLUMEUPKEY},
  { "TAG_VOLUMEUPMOD",       TAG_VOLUMEUPMOD},
  { "TAG_VOLUMEDOWNKEY",     TAG_VOLUMEDOWNKEY},
  { "TAG_VOLUMEDOWNMOD",     TAG_VOLUMEDOWNMOD},
  { "TAG_MUTEKEY",           TAG_MUTEKEY},
  { "TAG_MUTEMOD",           TAG_MUTEMOD},
  { "TAG_MIXERINITDELAY",    TAG_MIXERINITDELAY},
  { "TAG_VOLUME",            TAG_VOLUME },
  { "TAG_MUTE",              TAG_MUTE },
  { "TAG_MIXERCHANNELS",     TAG_MIXERCHANNELS },
  { "TAG_CDROMDEVICE",       TAG_CDROMDEVICE },
  { "TAG_EJECTCDKEY",        TAG_EJECTCDKEY },
  { "TAG_EJECTCDMOD",        TAG_EJECTCDMOD },
  { "TAG_EJECTCDKEYDELAY",   TAG_EJECTCDKEYDELAY },
  { "TAG_EJECTCD",           TAG_EJECTCD },
  { "TAG_SCRIPTBATCRITICAL", TAG_SCRIPTBATCRITICAL },
  { "TAG_SCRIPTPROFCHANGED", TAG_SCRIPTPROFCHANGED},
  { "TAG_SCRIPTHDSETUP",     TAG_SCRIPTHDSETUP},
  { "TAG_SLEEPKEY",          TAG_SLEEPKEY },
  { "TAG_SLEEPMOD",          TAG_SLEEPMOD },
  { "TAG_SLEEPKEYDELAY",     TAG_SLEEPKEYDELAY },
  { "TAG_GOTOSLEEP",         TAG_GOTOSLEEP },
  { "TAG_BWLFIRST",          TAG_BWLFIRST },
  { "TAG_BWLSECOND",         TAG_BWLSECOND },
  { "TAG_BWLLAST",           TAG_BWLLAST },
  { "TAG_CURRENTBWL",        TAG_CURRENTBWL },
  { "TAG_SLEEPINSECONDS",    TAG_SLEEPINSECONDS },
  { "TAG_CPULOADSLEEPLOCK",  TAG_CPULOADSLEEPLOCK },
  { "TAG_CPULOADMIN",        TAG_CPULOADMIN },
  { "TAG_CPULOADPERIOD",     TAG_CPULOADPERIOD },
  { "TAG_NETLOADSLEEPLOCK",  TAG_NETLOADSLEEPLOCK },
  { "TAG_NETLOADMIN",        TAG_NETLOADMIN },
  { "TAG_NETLOADPERIOD",     TAG_NETLOADPERIOD },
  { "TAG_NETLOADDEV",        TAG_NETLOADDEV },
  { "TAG_ONACSLEEP",         TAG_ONACSLEEP },
  { "TAG_ONACDIM",           TAG_ONACDIM },
  { "TAG_ONACBLANK",         TAG_ONACBLANK },
  { "TAG_ONACCOVERSLEEP",    TAG_ONACCOVERSLEEP },
  { "TAG_ONACTIMESLEEP",     TAG_ONACTIMESLEEP },
  { "TAG_ONACTIMEDIM",       TAG_ONACTIMEDIM },
  { "TAG_ONACTIMEHDOFF",     TAG_ONACTIMEHDOFF },
  { "TAG_ONBATTERYSLEEP",    TAG_ONBATTERYSLEEP },
  { "TAG_ONBATTERYDIM",      TAG_ONBATTERYDIM },
  { "TAG_ONBATTERYBLANK",     TAG_ONBATTERYBLANK },
  { "TAG_ONBATTERYCOVERSLEEP", TAG_ONBATTERYCOVERSLEEP },
  { "TAG_ONBATTERYTIMESLEEP", TAG_ONBATTERYTIMESLEEP },
  { "TAG_ONBATTERYTIMEDIM",  TAG_ONBATTERYTIMEDIM },
  { "TAG_ONBATTERYTIMEHDOFF",TAG_ONBATTERYTIMEHDOFF },
  { "TAG_EMERGENCYACTION",   TAG_EMERGENCYACTION },
  { "TAG_IDENTITY",          TAG_IDENTITY },
  { NULL, 0 }};

  static struct tagtable errortab[] = {
  { N_("Permission denied"),           E_PERM},
  { N_("Private Tag"),                 E_PRIVATE},
  { N_("File doesn't exist"),          E_NOEXIST},
  { N_("File not a charakter device"), E_NOCHAR},
  { N_("File not a block device"),     E_NOBLK},
  { N_("File not a file"),             E_NOFILE},
  { N_("Buffer overflow"),             E_BUFOVL},
  { N_("open error"),                  E_OPEN},
  { N_("format error"),                E_FORMAT},
  { N_("Messageport not available"),    E_MSGPORT},
  { N_("Server already running"),         E_TWICE},
  { N_("Help or version info"),             E_INFO},
  { N_("Unsecure script owner"),             E_USER},
  { N_("Script must be write-only by owner"), E_RIGHTS},
  { N_("read-only value"),					 E_NOWRITE},
  { N_("write-only value"),		  			E_NOREAD},
  { N_("function not supported"),		  E_NOSUPPORT},
  { NULL, 0 }};

struct moddata_peep {
  int timer10;
  int timer100;
  int timer1000;
  int secure;
} modbase_peep;

int
peep_init (struct tagitem *taglist)
{
	struct moddata_peep *base = &modbase_peep;

	base->timer10    = 0;
	base->timer100   = 0;
	base->timer1000  = 0;
	base->secure     = 0;

	register_function (KBDQUEUE, peep_keyboard);
#if 0
	register_function (MOUSEQUEUE, peep_mouse);
	register_function (T10QUEUE, peep_t10);
	register_function (T100QUEUE, peep_t100);
	register_function (T1000QUEUE, peep_t1000);
#endif
	register_function (CONFIGQUEUE, peep_config);
	register_function (QUERYQUEUE, peep_query);
	register_function (SECUREQUEUE, peep_secure);
	return 0;
}

int
peep_exit ()
{
  return 0;
}

void
peep_keyboard (struct tagitem *taglist)
{
	int code = (int) tagfind (taglist, TAG_KEYCODE, 0);
	int value = (int) tagfind (taglist, TAG_KEYREPEAT, 0);
	int mod = (int) tagfind (taglist, TAG_MODIFIER, 0);
	printf("\tKeyboard: code %3d, value %d, modifier %d\r", code, value, mod);
	fflush(stdout);
}

#if 0
void
peep_mouse (struct tagitem *taglist)
{
	int buttons = (int) tagfind (taglist, TAG_MOUSEBUTTONS, 0);
	int rel_x = (int) tagfind (taglist, TAG_MOUSERELX, 0);
	int rel_y = (int) tagfind (taglist, TAG_MOUSERELY, 0);
	int wheel = (int) tagfind (taglist, TAG_MOUSEWHEEL, -1);
	if (wheel == -1)
		printf("Mouse: buttons %x, rel-X %d, rel-Y %d\n", buttons, rel_x, rel_y);
	else
		printf("Mouse: buttons %x, rel-X %d, rel-Y %d, wheel %d\n", buttons, rel_x, rel_y, wheel);
}

void
peep_t10 (struct tagitem *taglist)
{
	struct moddata_peep *base = &modbase_peep;
	base->timer10++;
}

void
peep_t100 (struct tagitem *taglist)
{
	struct moddata_peep *base = &modbase_peep;
	base->timer100++;
}

void
peep_t1000 (struct tagitem *taglist)
{
	struct moddata_peep *base = &modbase_peep;
	base->timer1000++;
	printf ("timer10: %d   timer100: %d   timer1000: %d   secure: %d\n", base->timer10, base->timer100, base->timer1000, base->secure);
}
#endif

void
peep_ipc (struct tagitem *taglist)
{
	peep_taglist ("IPC", taglist);
}

void
peep_config (struct tagitem *taglist)
{
	peep_taglist ("Config", taglist);
}

void
peep_query (struct tagitem *taglist)
{
	peep_taglist ("Query", taglist);
}

void
peep_secure (struct tagitem *taglist)
{
	peep_taglist ("Secure", taglist);
}

void
peep_taglist (char *source, struct tagitem *taglist)
{
	struct tagtable *tt, *tt2;

	if (taglist != NULL)
		while (taglist->tag != TAG_END && taglist->tag != (TAG_TIMEREMAINING)) {
			tt = findttentry (tagtab,  taglist->tag & MASK_ERROR);
			if (tt == NULL) {
				printf (_("unknown tag\n"));
			} else if (tt->tag == TAG_SKIP) {
				tt2 = findttentry (tagtab,  taglist->data);
				if (tt2 != NULL)
					printf ("%-10s: %-25s: %s                   \n", source, tt->tagname, tt2->tagname);
				else
					printf ("%-10s: %-25s: ???                   \n", source, tt->tagname);
			} else if (taglist->tag & FLG_ERROR) {
				tt2 = findttentry (errortab,  taglist->data);
				if (tt2 != NULL)
					printf ("%-10s: %-25s: %s                   \n", source, tt->tagname, tt2->tagname);
				else
					printf ("%-10s: %-25s: ???                   \n", source, tt->tagname);
			} else if (tt->tag & FLG_STRING) {
					printf ("%-10s: %-25s: \"%s\"                   \n", source, tt->tagname, (char *) taglist->data);
			} else {
					printf ("%-10s: %-25s: %ld                    \n", source, tt->tagname, taglist->data);
			}
			taglist++;
		}
}

struct tagtable *
findttentry (struct tagtable *ttab, long tag)
{
	while (ttab->tag != 0) {
		if (ttab->tag == tag)
			return ttab;
		ttab++;
	}
	return NULL;
}

#endif /* Debug */
