#ifndef INCLUDE_MODULE_PMAC_H
#define INCLUDE_MODULE_PMAC_H
/* ----------------------------------------------------------------------------
 * module_PMAC.h
 * prototypes and definitions for the laptop module pmac.
 *
 * Copyright 2002 Matthias Grimm (joker@cymes.de).
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version
 * 2 of the License, or (at your option) any later version.
 * ----------------------------------------------------------------------------*/

#include <pbbtaglist.h>

#define ADB_BUFSIZE			32      /* Size of the ADB buffer, constant,
                                            must not be changed !! */

#define OHARE_PMU			9

#define BACKLIGHTMAX		15      /* from asm/backlight.h */
#define MAX_BATTERIES		2

#define DEFAULT_PMU			"/dev/pmu"
#define DEFAULT_ADB			"/dev/adb"
#define DEFAULT_BATLOG		"/var/lib/pbbuttons/%03d_battery.log"
#define DEFAULT_BATCYCLE	"/var/lib/pbbuttons/battery.cycle"

/* definitions for input device config */
#define TRACKPAD_NOTAP	0
#define TRACKPAD_TAP	1
#define TRACKPAD_DRAG	2
#define TRACKPAD_LOCK	3
#define TRACKPAD_MASK	3

#define KEYBOARD_FNTOP	1
#define KEYBOARD_MASK	1

#ifndef PMU_IOC_GRAB_BACKLIGHT
#define PMU_IOC_GRAB_BACKLIGHT	_IOR('B', 6, 0)
#endif

struct modflags_pmac {
	unsigned int coveropen :1;
	unsigned int ac_power:1;
	unsigned int sleepsupported :1;
	unsigned int set_tpmode:1;               /* delayed function triggered */
	unsigned int set_kbdmode:1;            /* delayed function triggered */
	unsigned int goto_sleep:1;                /* delayed function triggered */
	unsigned int bat_present:MAX_BATTERIES;
	unsigned int bat_charging:MAX_BATTERIES;
	unsigned int :0;
};

/* public prototypes */
int pmac_init (struct tagitem *taglist);
int pmac_exit ();

/* private prototypes */
void pmac_keyboard (struct tagitem *taglist);
void pmac_secure (struct tagitem *taglist);
void pmac_query (struct tagitem *taglist);
void pmac_configure (struct tagitem *taglist);
int pmac_handle_tags (int cfgure, struct tagitem *taglist);
void pmac_pmu_handler (int fd);
void pmac_timer (struct tagitem *taglist);

void pmac_update_batteryinfo ();
void activate_sleepmode ();
void pmac_identify (char* buffer, int pmu);

int keyboard_get_config(int fd);
void keyboard_set_config(int fd, int config);
int keyboard_get_dev(int fd);
int trackpad_get_config(int fd);
void trackpad_set_config(int fd, int config);
int trackpad_get_dev(int fd);
void trackpad_set_prgmode(int fd, int dev, int mode);
int send_pmu_request(int fd, unsigned char *buffer, int params, ...);
int adb_read_reg(int fd, unsigned char *ADBBuffer, int dev, int reg);
int adb_write_reg(int fd, unsigned char *ADBBuffer, int len, int dev, int reg);
int get_pmu_version(int fd);
int get_button(int fd, int button);
#ifndef WITH_PMUD
int get_cover_position(int fd);
#endif

#ifdef BATLOG
int batlog_setup();
void batlog_save();
void batlog_write(long timerem);
#endif

#endif    /* INCLUDE_MODULE_PMAC_H */
