/* --------------------------------------------------------------------------
 * module_system.c
 * code for general system functions
 *
 * Copyright 2002 Matthias Grimm (joker@cymes.de).
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version
 * 2 of the License, or (at your option) any later version.
 * -------------------------------------------------------------------------*/
#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#include <string.h>
#include <pbb.h>

#include "gettext_macros.h"
#include "input_manager.h"
#include "module_system.h"

extern volatile int saveconfig;

struct moddata_system {
	char *userallowed;  /* remember system tag for config file saving */
	int	autorescan;
	int rescantimer;
} modbase_system;

int
system_init (struct tagitem *taglist)
{
	struct moddata_system *base = &modbase_system;
	static char buf_userallowed[STDBUFFERLEN];
	int rc;

	base->autorescan = 0;   /* disable by default */
	base->rescantimer = 0;
	base->userallowed = buf_userallowed;
	base->userallowed[0] = '\0';

	if ((rc = system_handle_tags (MODE_CONFIG, taglist))) /* interprete configfile */
		return rc;                 /* critcal error occured, abort program */

	ipc_protect_tag (TAG_SAVECONFIG);  /* tags only for privileged use */
	register_function (QUERYQUEUE, system_query);
	register_function (CONFIGQUEUE, system_configure);
	register_function (T1000QUEUE, system_timer1000);
	return 0;
}

int
system_exit ()
{
	return 0;
}

void
system_query (struct tagitem *taglist)
{
	system_handle_tags (MODE_QUERY, taglist);
}

void
system_configure (struct tagitem *taglist)
{
	system_handle_tags (MODE_CONFIG, taglist);
}

int
system_handle_tags (int cfgure, struct tagitem *taglist)
{
	struct moddata_system *base = &modbase_system;
	static char *version = VERSION;
	int rc = 0;

	while (taglist->tag != TAG_END) {
		switch (taglist->tag) {
		case TAG_SAVECONFIG:
			if (cfgure)	saveconfig = 1;
			else		tagerror (taglist, E_NOREAD);
			break;
		case TAG_USERALLOWED:  /* private tag */
			if (cfgure)	strncpy(base->userallowed, (char*) taglist->data, STDBUFFERLEN);
			else		taglist->data = (long) base->userallowed;
			break;
		case TAG_VERSION:
			if (cfgure)	tagerror (taglist, E_NOWRITE);
			else		taglist->data = (long) version;
			break;
		case TAG_AUTORESCAN:
			if (cfgure)	base->autorescan = taglist->data;
			else		taglist->data = (long) base->autorescan;
			break;
		}
		taglist++;
	}
	return rc;
}

void
system_timer1000 (struct tagitem *taglist)
{
	struct moddata_system *base = &modbase_system;

	if (base->autorescan) {
		base->rescantimer++;
		if (base->rescantimer > RESCANTIME) {
			base->rescantimer = 0;
			scan_for_kbdevdevs();
		}
	}
}

